## The effects of rolloff manipulation encoded as 0 (down) / 0.5 (original) / 1 (up) 
# on valence / arousal / dominance ratings

setwd('/home/allgoodguys/Documents/Studying/Lund_PhD/epistles/014_rolloff/soundgen_ac-cor-em')

library(ggplot2)
library(lme4)
library(brms)
source('/home/allgoodguys/Documents/Soft_comp/R/ggplot_multiple.R')

# load data
df = read.csv('data/rn.csv')  # final sample

ac = read.csv('data/rn_sumtab.csv')
myVars = c('specCentroid_median') 
df[, myVars] = ac[match(df$sound, ac$sound1), myVars]

df$log_sc = log10(df$specCentroid_median)
# summary(df$rolloff)

if (FALSE) {
  ## what's the average effect of rolloff on spectral centroid?
  a = aggregate(specCentroid_median ~ sound + sound_base + rolloff, df, mean)
  b = data.frame(sound_base = unique(df$sound_base),
                 orig_minus_down = NA,
                 up_minus_orig = NA)
  for (i in 1:nrow(b)) {
    temp = a[a$sound_base == b$sound_base[i], ]
    b$orig_minus_down[i] = temp$specCentroid_median[temp$rolloff == 'orig'] -
      temp$specCentroid_median[temp$rolloff == 'down'] 
    b$up_minus_orig[i] = temp$specCentroid_median[temp$rolloff == 'up'] -
      temp$specCentroid_median[temp$rolloff == 'orig'] 
  }
  apply(b, 2, mean)
  apply(b, 2, sd)
}



### VALENCE
df_val = df[!is.na(df$valence), ]
df_val$valence_norm = (df_val$valence + 1.01)/2.02  # range 0 to 1, exclusive

## without controlling for specCentroid_median
if (FALSE) {
  # simple plots
  a = aggregate(valence_norm ~ rolloff_dB + sound_base, df_val, mean)
  ggplot(a, aes(x = rolloff_dB, y = valence_norm)) +
    geom_point() +
    geom_smooth()
  
  boxplot(valence_norm ~ rolloff_dB, df_val)
  boxplot(valence_norm ~ rolloff, df_val)
  
  a1 = aggregate(valence_norm ~ log_sc + sound_base, df_val, mean)
  ggplot(a1, aes(x = log_sc, y = valence_norm)) +
    geom_point() +
    geom_smooth()
  
  # model selection
  mod_val0 = brm(valence_norm ~ rolloff_dB + (1|id) + (1|sound_base) + (1|sound), data = df_val, family = 'beta', cores = 2, chains = 2, iter = 4000, warmup = 500, prior = set_prior("normal(0,5)"))
  mod_val = brm(valence_norm ~ rolloff01 + (1|id) + (1|sound_base) + (1|sound), data = df_val, family = 'beta', cores = 2, chains = 2, iter = 4000, warmup = 500, prior = set_prior("normal(0,5)"))
  mod_val1 = brm(valence_norm ~ rolloff01 + (1|id) + (rolloff01|sound_base) + (1|sound), data = df_val, family = 'beta', cores = 2, chains = 2, iter = 4000, warmup = 500, prior = set_prior("normal(0,5)"))
  mod_val2 = brm(valence_norm ~ rolloff01 + (1|id) + (rolloff01|sound_base), data = df_val, family = 'beta', cores = 2, chains = 2, iter = 4000, warmup = 500, prior = set_prior("normal(0,5)"))
  
  waic(mod_val0, mod_val, mod_val1, mod_val2)  # mod_val2 is definitely worse, otherwise a close call
  # saveRDS(mod_val, 'models/mod_val.RDS')  # saveRDS(mod_val, 'models/mod_val_all.RDS')
}

mod_val = readRDS('models/mod_val.RDS')  # mod_val = readRDS('models/mod_val_all.RDS')
# plot(mod_val)
# brms::pp_check(mod_val)
# brms::marginal_effects(mod_val)
# summary(mod_val)

fit_val = fitted(mod_val, newdata = data.frame(rolloff01 = c(0, 1)), re_formula = NA, summary = FALSE)
soundgen:::reportCI(quantile((fit_val[, 2] - fit_val[, 1]) * 100, probs = c(.5, .025, .975)), digits = 1)  # -3.4 [-6.3, -0.4]


## controlling for specCentroid_median
# mod_val_sc = brm(valence_norm ~ rolloff01 + log_sc + (1|id) + (1|sound_base) + (1|sound), data = df_val, family = 'beta', cores = 2, chains = 2, iter = 4000, warmup = 500, prior = set_prior("normal(0,5)"))
# saveRDS(mod_val_sc, 'models/mod_val_sc.RDS')  #  saveRDS(mod_val_sc, 'models/mod_val_sc_all.RDS')
mod_val_sc = readRDS('models/mod_val_sc.RDS')  # mod_val_sc = readRDS('models/mod_val_sc_all.RDS')
# plot(mod_val_sc)
# brms::pp_check(mod_val_sc)
# brms::marginal_effects(mod_val_sc)
# summary(mod_val_sc)

fit_val_sc = fitted(mod_val_sc, newdata = data.frame(rolloff01 = c(0, 1), log_sc = mean(df_val$log_sc)), re_formula = NA, summary = FALSE)
soundgen:::reportCI(quantile((fit_val_sc[, 2] - fit_val_sc[, 1]) * 100, probs = c(.5, .025, .975)), digits = 1)  # -3.0 [-6.6, 0.5]

fit_val_sc1 = fitted(mod_val_sc, newdata = data.frame(rolloff01 = 0, log_sc = c(min(df_val$log_sc), max(df_val$log_sc))), re_formula = NA, summary = FALSE)
soundgen:::reportCI(quantile((fit_val_sc1[, 2] - fit_val_sc1[, 1]) * 100, probs = c(.5, .025, .975)), digits = 1)  # -2.6 [-17.8, 11.7]


### AROUSAL
df_ar = df[!is.na(df$arousal), ]
df_ar$arousal_norm = (df_ar$arousal + 1.01)/2.02  # range 0 to 1, exclusive

# mod_ar = brm(arousal_norm ~ rolloff01 + (1|id) + (1|sound_base) + (1|sound), data = df_ar, family = 'beta', cores = 2, chains = 2, iter = 4000, warmup = 500, prior = set_prior("normal(0,5)"))
# saveRDS(mod_ar, 'models/mod_ar.RDS')  # saveRDS(mod_ar, 'models/mod_ar_all.RDS')
mod_ar = readRDS('models/mod_ar.RDS')  # mod_ar = readRDS('models/mod_ar_all.RDS')
# summary(mod_ar)

fit_ar = fitted(mod_ar, newdata = data.frame(rolloff01 = c(0, 1)), re_formula = NA, summary = FALSE)
soundgen:::reportCI(quantile((fit_ar[, 2] - fit_ar[, 1]) * 100, probs = c(.5, .025, .975)), digits = 1)  # 6.9 [3.6, 10.1]


# mod_ar_sc = brm(arousal_norm ~ rolloff01 + log_sc + (1|id) + (1|sound_base) + (1|sound), data = df_ar, family = 'beta', cores = 2, chains = 2, iter = 4000, warmup = 500, prior = set_prior("normal(0,5)"))
# saveRDS(mod_ar_sc, 'models/mod_ar_sc.RDS')  #  saveRDS(mod_ar_sc, 'models/mod_ar_sc_all.RDS')
mod_ar_sc = readRDS('models/mod_ar_sc.RDS')  # mod_ar_sc = readRDS('models/mod_ar_sc_all.RDS')
# summary(mod_ar_sc)

fit_ar_sc = fitted(mod_ar_sc, newdata = data.frame(rolloff01 = c(0, 1), log_sc = mean(df_ar$log_sc)), re_formula = NA, summary = FALSE)
soundgen:::reportCI(quantile((fit_ar_sc[, 2] - fit_ar_sc[, 1]) * 100, probs = c(.5, .025, .975)), digits = 1)  # 5.3 [1.0, 9.3]

fit_ar_sc1 = fitted(mod_ar_sc, newdata = data.frame(rolloff01 = 0, log_sc = c(min(df_ar$log_sc), max(df_ar$log_sc))), re_formula = NA, summary = FALSE)
soundgen:::reportCI(quantile((fit_ar_sc1[, 2] - fit_ar_sc1[, 1]) * 100, probs = c(.5, .025, .975)), digits = 1)  # 11.3 [-6.0, 29.0]




### DOMINANCE
df_dom = df[!is.na(df$dominance), ]
df_dom$dominance_norm = (df_dom$dominance + 1.01)/2.02  # range 0 to 1, exclusive

# mod_dom = brm(dominance_norm ~ rolloff01 + (1|id) + (1|sound_base) + (1|sound), data = df_dom, family = 'beta', cores = 2, chains = 2, iter = 4000, warmup = 500, prior = set_prior("normal(0,5)"))
# saveRDS(mod_dom, 'models/mod_dom.RDS')  # saveRDS(mod_dom, 'models/mod_dom_all.RDS')
mod_dom = readRDS('models/mod_dom.RDS')  # mod_dom = readRDS('models/mod_dom_all.RDS')
# summary(mod_dom)

fit_dom = fitted(mod_dom, newdata = data.frame(rolloff01 = c(0, 1)), re_formula = NA, summary = FALSE)
soundgen:::reportCI(quantile((fit_dom[, 2] - fit_dom[, 1]) * 100, probs = c(.5, .025, .975)), digits = 1)  # 2.8 [0.4, 5.1]


# mod_dom_sc = brm(dominance_norm ~ rolloff01 + log_sc + (1|id) + (1|sound_base) + (1|sound), data = df_dom, family = 'beta', cores = 2, chains = 2, iter = 4000, warmup = 500, prior = set_prior("normal(0,5)"))
# saveRDS(mod_dom_sc, 'models/mod_dom_sc.RDS')  #  saveRDS(mod_dom_sc, 'models/mod_dom_sc_all.RDS')
mod_dom_sc = readRDS('models/mod_dom_sc.RDS')  # mod_dom_sc = readRDS('models/mod_dom_sc_all.RDS')
# summary(mod_dom_sc)

fit_dom_sc = fitted(mod_dom_sc, newdata = data.frame(rolloff01 = c(0, 1), log_sc = mean(df_dom$log_sc)), re_formula = NA, summary = FALSE)
soundgen:::reportCI(quantile((fit_dom_sc[, 2] - fit_dom_sc[, 1]) * 100, probs = c(.5, .025, .975)), digits = 1)  # 2.1 [-0.8, 5.1]

fit_dom_sc1 = fitted(mod_dom_sc, newdata = data.frame(rolloff01 = 0, log_sc = c(min(df_dom$log_sc), max(df_dom$log_sc))), re_formula = NA, summary = FALSE)
soundgen:::reportCI(quantile((fit_dom_sc1[, 2] - fit_dom_sc1[, 1]) * 100, probs = c(.5, .025, .975)), digits = 1)  # 4.9 [-6.6, 16.2]


if (FALSE) {
  ### Multivariate model w/o spectral centroid
  df = df[!is.na(df$valence) & !is.na(df$arousal) & !is.na(df$dominance), ]
  df$valence_norm = (df$valence + 1.01)/2.02
  df$arousal_norm = (df$arousal + 1.01)/2.02 
  df$dominance_norm = (df$dominance + 1.01)/2.02 
  
  mod = brm(mvbind(valence_norm, arousal_norm, dominance_norm) ~ rolloff01 + (1|id) + (1|sound_base) + (1|sound), data = df, family = 'beta', cores = 4, chains = 4, iter = 4000, warmup = 500, prior = set_prior("normal(0, 5)"))
  mod1 = brm(mvbind(valence_norm, arousal_norm, dominance_norm) ~ rolloff01 + (1|p|id) + (1|q|sound_base) + (1|r|sound), data = df, family = 'beta', cores = 4, chains = 4, iter = 4000, warmup = 500, prior = set_prior("normal(0, 5)"))  # model group-level intercepts as correlated
  waic(mod, mod1)  # same
  summary(mod1)
  # pp_check(mod, resp = 'valencenorm')
  fit = fitted(mod1, newdata = data.frame(rolloff01 = c(0, 1)), re_formula = NA, summary = FALSE)  # 3D array
  quantile((fit[, 2, 1] - fit[, 1, 1]) * 100, probs = c(.5, .025, .975))
  quantile((fit[, 2, 2] - fit[, 1, 2]) * 100, probs = c(.5, .025, .975))
  quantile((fit[, 2, 3] - fit[, 1, 3]) * 100, probs = c(.5, .025, .975))
  # very similar to univariate
}
