# The effect of other acoustic characteristics on the effect of rolloff manipulation (interaction effects)

setwd('/home/allgoodguys/Documents/Studying/Lund_PhD/epistles/014_rolloff/soundgen_ac-cor-em')

library(ggplot2)
library(lme4)
library(brms)

# load data
df = read.csv('data/rn.csv')  # final sample of "clean" data
# df = read.csv('data/rn_all.csv') # w/o excluding any participants

df = df[!is.na(df$rolloff), ]
summary(df$rolloff)

ac = read.csv('data/rn_sumtab.csv')
colnames(ac)
myVars = c('sound_master', 'sex', 'dur', 'rolloff_dB', 'pitchMedian', 'sylLenMedian', 'sylLenMax', 'loudness_median', 'loudness_max', 'specCentroid_median') 
df[, myVars] = ac[match(df$sound, ac$sound1), myVars]

if (FALSE) {
  ## Simple plotting
  v = c('sex', 'dur', 'sylLenMedian', 'sylLenMax', 'pitchMedian', 'loudness_median', 'loudness_max')
  
  # Effect of v on valence
  a_val = aggregate(valence ~ rolloff + sound, df, mean)
  a_val$sound_base = df$sound_base[match(a_val$sound, df$sound)]
  out = data.frame(sound_base = unique(df$sound_base), valDif = NA)
  for (i in 1:nrow(out)) {
    temp = a_val[a_val$sound_base == out$sound_base[i], ]
    out$valDif[i] = temp$valence[temp$rolloff == 'up'] - temp$valence[temp$rolloff == 'down']
  }
  out[, v] = df[match(out$sound_base, df$sound_base), v]
  
  ggplot(out, aes(x = dur, y = valDif)) + geom_point() + geom_smooth() + theme_bw()
  ggplot(out, aes(x = sylLenMedian, y = valDif)) + geom_point() + geom_smooth() + theme_bw()
  ggplot(out, aes(x = sylLenMax, y = valDif)) + geom_point() + geom_smooth() + theme_bw()
  ggplot(out, aes(x = sex, y = valDif)) + geom_point() + geom_boxplot() + theme_bw()
  ggplot(out, aes(x = loudness_median, y = valDif)) + geom_point() + geom_smooth() + theme_bw()
  ggplot(out, aes(x = loudness_max, y = valDif)) + geom_point() + geom_smooth() + theme_bw()
  # looks like only sex has an effect (negative effect of rolloff slightly stronger for males)
  
  
  # Effect of v on arousal
  a_ar = aggregate(arousal ~ rolloff + sound, df, mean)
  a_ar$sound_base = df$sound_base[match(a_ar$sound, df$sound)]
  out = data.frame(sound_base = unique(df$sound_base), valDif = NA)
  for (i in 1:nrow(out)) {
    temp = a_ar[a_ar$sound_base == out$sound_base[i], ]
    out$valDif[i] = temp$arousal[temp$rolloff == 'up'] - temp$arousal[temp$rolloff == 'down']
  }
  out[, v] = df[match(out$sound_base, df$sound_base), v]
  
  ggplot(out, aes(x = dur, y = valDif)) + geom_point() + geom_smooth() + theme_bw()
  ggplot(out, aes(x = sylLenMedian, y = valDif)) + geom_point() + geom_smooth() + theme_bw()
  ggplot(out, aes(x = sylLenMax, y = valDif)) + geom_point() + geom_smooth() + theme_bw()
  ggplot(out, aes(x = sex, y = valDif)) + geom_point() + geom_boxplot() + theme_bw()
  # possibly more pronounced for males
  
  
  # Effect of v on dominance
  a_dom = aggregate(dominance ~ rolloff + sound, df, mean)
  a_dom$sound_base = df$sound_base[match(a_dom$sound, df$sound)]
  out = data.frame(sound_base = unique(df$sound_base), valDif = NA)
  for (i in 1:nrow(out)) {
    temp = a_dom[a_dom$sound_base == out$sound_base[i], ]
    out$valDif[i] = temp$dominance[temp$rolloff == 'up'] - temp$dominance[temp$rolloff == 'down']
  }
  out[, v] = df[match(out$sound_base, df$sound_base), v]
  
  ggplot(out, aes(x = dur, y = valDif)) + geom_point() + geom_smooth() + theme_bw()
  ggplot(out, aes(x = sylLenMedian, y = valDif)) + geom_point() + geom_smooth() + theme_bw()
  ggplot(out, aes(x = sylLenMax, y = valDif)) + geom_point() + geom_smooth() + theme_bw()
  ggplot(out, aes(x = sex, y = valDif)) + geom_point() + geom_boxplot() + theme_bw()
  # might be an effect for males only, plus possibly smth to do with dur (?)
}




## Models
df$logPitch = log(df$pitchMedian)
df$logSpecCen = log(df$specCentroid_median)
myPreds = c('sylLenMax', 'sylLenMedian', 'dur', 'loudness_median', 'loudness_max', 'logPitch', 'logSpecCen', 'sex')

## VALENCE
df_val = df[!is.na(df$valence), ]
test_val = data.frame(predictor = myPreds, p = NA)
for (i in 1:nrow(test_val)) {
  f = formula(paste('valence ~ rolloff01 * ', test_val$predictor[i], '+ (1|id) + (1|sound_base) + (1|sound)'))
  mod = lmer(f, data = df_val)
  # summary(mod)
  d = drop1(mod, test = 'Chisq')
  test_val$p[i] = d$`Pr(Chi)`[2]
}
test_val$bonfer = test_val$p < (0.05 / length(myPreds))
test_val  # nothing for valence


## AROUSAL
df_ar = df[!is.na(df$arousal), ]
test_ar = data.frame(predictor = myPreds, p = NA)
for (i in 1:nrow(test_ar)) {
  f = formula(paste('arousal ~ rolloff01 * ', test_ar$predictor[i], '+ (1|id) + (1|sound_base) + (1|sound)'))
  mod = lmer(f, data = df_ar)
  # summary(mod)
  d = drop1(mod, test = 'Chisq')
  test_ar$p[i] = d$`Pr(Chi)`[2]
}
test_ar$bonfer = test_ar$p < (0.05 / length(myPreds))
test_ar  # nothing for arousal


## DOMINANCE
df_dom = df[!is.na(df$dominance), ]
test_dom = data.frame(predictor = myPreds, p = NA)
for (i in 1:nrow(test_dom)) {
  f = formula(paste('dominance ~ rolloff01 * ', test_dom$predictor[i], '+ (1|id) + (1|sound_base) + (1|sound)'))
  mod = lmer(f, data = df_dom)
  d = drop1(mod, test = 'Chisq')
  test_dom$p[i] = d$`Pr(Chi)`[2]
}
test_dom$bonfer = test_dom$p < (0.05 / length(myPreds))
test_dom  # nothing for dominance

# mod_sylLen = lmer(dominance ~ rolloff * sylLenMedian + (1|id) + (1|sound_base) + (1|sound), data = df_dom)
# summary(mod_sylLen)
