## The effect of rolloff manipulations on spectral centroid

setwd('/home/allgoodguys/Documents/Studying/Lund_PhD/epistles/009_nonlin-rolloff/soundgen_ac-cor-em/analysis')

library(ggplot2)
library(lme4)
library(brms)
library(soundgen)

# load data
df = read.csv('data/rn.csv')  # final sample of "clean" data
# df = read.csv('data/rn_all.csv') # w/o excluding any participants

df = droplevels(df[!is.na(df$rolloff), ])

ac = read.csv('data/rn_sumtab.csv')
myVars = c('specCentroid_median') 
df[, myVars] = ac[match(df$sound, ac$sound1), myVars]

df$log_sc = log10(df$specCentroid_median)
summary(df$rolloff)

## what's the average effect of rolloff on spectral centroid?
a = aggregate(specCentroid_median ~ sound + sound_base + rolloff, df, mean)
b = data.frame(sound_base = unique(df$sound_base),
               orig_minus_down = NA,
               up_minus_orig = NA)
for (i in 1:nrow(b)) {
  temp = a[a$sound_base == b$sound_base[i], ]
  b$orig_minus_down[i] = temp$specCentroid_median[temp$rolloff == 'orig'] -
    temp$specCentroid_median[temp$rolloff == 'down'] 
  b$up_minus_orig[i] = temp$specCentroid_median[temp$rolloff == 'up'] -
    temp$specCentroid_median[temp$rolloff == 'orig'] 
}
apply(b, 2, mean)
apply(b, 2, sd)

a$sound_base = as.factor(a$sound_base)
# mod_meanFreq = brm(specCentroid_median ~ rolloff + (1|sound_base), data = a, family = 'lognormal', cores = 2, chains = 2, iter = 8000, prior = set_prior('normal(0, 5)'))
# saveRDS(mod_meanFreq, 'models/mod_meanFreq.RDS')
mod_meanFreq = readRDS('models/mod_meanFreq.RDS')
summary(mod_meanFreq)
plot(mod_meanFreq)

newdata = data.frame(rolloff = levels(df$rolloff))
f = fitted(mod_meanFreq, newdata = newdata, re_formula = NA, summary = FALSE)
soundgen:::reportCI(quantile(f[, 2] - f[, 1], probs = c(.5, .025, .975)), 0)  # orig - down
soundgen:::reportCI(quantile(f[, 3] - f[, 2], probs = c(.5, .025, .975)), 0)  # up - orig

# plot
colnames(f) = levels(df$rolloff)
df_plot_rolloff = data.frame(rolloff = c('Weaker harmonics', 'Stronger harmonics'), fit = NA, lwr = NA, upr = NA)
df_plot_rolloff[1, 2:4] = quantile(f[, 1] - f[, 2], probs = c(.5, .025, .975))
df_plot_rolloff[2, 2:4] = quantile(f[, 3] - f[, 2], probs = c(.5, .025, .975))
df_plot_rolloff$rolloff = factor(df_plot_rolloff$rolloff, levels = c('Weaker harmonics', 'Stronger harmonics'))

ggplot(df_plot_rolloff, aes(x = rolloff, y = fit, ymin = lwr, ymax = upr)) +
  geom_point(size = 3) +
  geom_errorbar(width = .2) +
  geom_hline(yintercept = 0, linetype = 2) +
  xlab('') + ylab('Change in spectral centroid, Hz') +
  ggtitle('B. Spectral centroid') +
  theme_bw() +
  theme(panel.grid = element_blank())
# ggsave(filename = 'pix/hnr_per_nonlin.jpg', width = 8, height = 8, units = 'cm', dpi = 300, scale = 1)

