# Acoustic characteristics of stimuli
setwd('/home/allgoodguys/Documents/Studying/Lund_PhD/epistles/014_rolloff/soundgen_confirm')

## summary table of stimuli
ac = read.csv('data/sc_acoustics.csv')
colnames(ac)

st = aggregate(sound_base ~ callType, ac, function(x) length(unique(x)))
br = aggregate(breathing ~ callType, ac, function(x) sum(x) / 3)[, 2]  # /3 b/c three sounds per prototype
st$sound_base = paste0(st$sound_base, ' (', br, ')')
sex_temp = aggregate(sound_base ~ callType + sex, ac, function(x) length(unique(x)))
st$sex = aggregate(sound_base ~ callType, sex_temp, function(x) paste(x, collapse = '/'))[, 2]

rolloff_mean = round(aggregate(rolloff_dB ~ callType, ac, function(x) mean(x[x > 0]))[, 2], 1)
rolloff_min = aggregate(rolloff_dB ~ callType, ac, function(x) min(x[x > 0]))[, 2]
rolloff_max = aggregate(rolloff_dB ~ callType, ac, function(x) max(x[x > 0]))[, 2]
st$rolloff = paste0('±', rolloff_mean, ' [', rolloff_min, ', ', rolloff_max, ']')
v = c('duration', 'sylLen_median', 'pitchMedian', 'HNR_median', 'specCentroid_median')
nDigits = c(1, 1, 0, 1, 1)

for (i in 1:length(v)) {
  myMean = aggregate(get(v[i]) ~ callType, ac, function(x) round(mean(x), nDigits[i]))[, 2]
  myRange = aggregate(get(v[i]) ~ callType, ac, function(x) round(range(x), nDigits[i]))[, 2]
  myRange = apply(myRange, 1, function(x) paste0('[', x[1], ', ', x[2], ']'))
  st[, v[i]] = paste(myMean, myRange)
}
st
write.csv(st, 'data/sum_stimuli.csv')


### exploring the results

library(DirichletReg) 

df = read.csv('rc.csv')[, -1]
# df_full = read.csv('rc_full.csv')[, -1]

gs = read.csv('gs.csv')[, -1]
l = read.csv('l.csv')[, -1]
mgg = read.csv('mgg.csv')[, -1]
sr = read.csv('sr.csv')[, -1]

# exclude rolloff condition
if (removeRolloff) {
  gs = gs[!gs$cond %in% c('R-', 'R+'), ]
  l = l[!l$cond %in% c('R-', 'R+'), ]
  mgg = mgg[!mgg$cond %in% c('R-', 'R+'), ]
  sr = sr[!sr$cond %in% c('R-', 'R+'), ]
}

# ratings per sound
summary(aggregate(time ~ sound, df, length)[, 2])  
summary(aggregate(time ~ sound, df_full, length)[, 2]) 

# n of stimuli per call type and condition
aggregate(sound ~ call + cond, df, function(x) length(unique(x)))
a = aggregate(subFreq ~ sound_base, df, function(x) length(unique(x)))
table(a$subFreq)

a = aggregate(time ~ id, df, function(x) sum(x) / 60000)  # 5-14 min
mean(a$time)


### Reliability of ratings for each call type
weights = list(
  gs = c('Hurt_weight', 'Pleased_weight', 'Surprised_weight'),
  l = c('Amused_weight', 'Polite_weight', 'Evil_weight'),
  mgg = c('Effortful_weight', 'Hurt_weight', 'Pleased_weight'),
  sr = c('Afraid_weight', 'Aggressive_weight', 'Pleased_weight')
)

library(ICC)
# gasps
boxplot(get(weights$gs[1]) ~ sound, gs)
boxplot(get(weights$gs[2]) ~ sound, gs)
mean(c(
  ICC::ICCest(x = sound, y = weights$gs[1], data = gs)$ICC,
  ICC::ICCest(x = sound, y = weights$gs[2], data = gs)$ICC,
  ICC::ICCest(x = sound, y = weights$gs[3], data = gs)$ICC
))

boxplot(coordX_st ~ sound, gs)
boxplot(coordY_st ~ sound, gs)
mean(c(
  ICC::ICCest(x = sound, y = coordX_st, data = gs)$ICC,
  ICC::ICCest(x = sound, y = coordY_st, data = gs)$ICC
))
# modX = anova(aov(coordX_st ~ sound, gs))
# ICC is basically a normalized proportion of variance in y (coordX_st) explained by x (sound)

# laughs
mean(c(
  ICC::ICCest(x = sound, y = weights$l[1], data = l)$ICC,
  ICC::ICCest(x = sound, y = weights$l[2], data = l)$ICC,
  ICC::ICCest(x = sound, y = weights$l[3], data = l)$ICC
))
mean(c(
  ICC::ICCest(x = sound, y = coordX_st, data = l)$ICC,
  ICC::ICCest(x = sound, y = coordY_st, data = l)$ICC
))

# moans
mean(c(
  ICC::ICCest(x = sound, y = weights$mgg[1], data = mgg)$ICC,
  ICC::ICCest(x = sound, y = weights$mgg[2], data = mgg)$ICC,
  ICC::ICCest(x = sound, y = weights$mgg[3], data = mgg)$ICC
))
mean(c(
  ICC::ICCest(x = sound, y = coordX_st, data = mgg)$ICC,
  ICC::ICCest(x = sound, y = coordY_st, data = mgg)$ICC
))

# screams
mean(c(
  ICC::ICCest(x = sound, y = weights$sr[1], data = sr)$ICC,
  ICC::ICCest(x = sound, y = weights$sr[2], data = sr)$ICC,
  ICC::ICCest(x = sound, y = weights$sr[3], data = sr)$ICC
))
mean(c(
  ICC::ICCest(x = sound, y = coordX_st, data = sr)$ICC,
  ICC::ICCest(x = sound, y = coordY_st, data = sr)$ICC
))




### DESCRIPTIVES

## effect of rolloff on meanFreq etc
boxplot(meanFreq_median ~ cond, droplevels(df[df$cond %in% c('O', 'R-', 'R+'), ]))
boxplot(meanFreq_median ~ cond + call, droplevels(df[df$cond %in% c('O', 'R-', 'R+'), ])) # NB: for gasps the effect of rolloff on meanFreq is actually reversed b/c of breathign
boxplot(peakFreq_median ~ cond, droplevels(df[df$cond %in% c('O', 'R-', 'R+'), ]))
# boxplot(meanFreq_median ~ cond, df)

## effect of nonlin on HNR
boxplot(HNR_median ~ cond, df)

# gasps
gs = read.csv('gs.csv')[, -1]
# colnames(gs)

boxplot(Surprised_weight ~ cond, gs)
boxplot(Hurt_weight ~ cond, gs)
boxplot(Pleased_weight ~ cond, gs)

gs_dir = DR_data(gs[, c('Hurt_weight', 'Pleased_weight', 'Surprised_weight')])
plot(gs_dir)
mod_gs = DirichReg(gs_dir ~ cond, gs)
newdata = data.frame(cond = levels(gs$cond))
pred_gs = DR_data(predict(mod_gs, newdata))
rownames(pred_gs) = levels(gs$cond)
colnames(pred_gs) = c('Hurt_weight', 'Pleased_weight', 'Surprised_weight')
plot(pred_gs)
pred_gs


# laughs
l = read.csv('l.csv')[, -1]
# colnames(l)

boxplot(Amused_weight ~ cond, l)
boxplot(Polite_weight ~ cond, l)
boxplot(Evil_weight ~ cond, l)

l_dir = DR_data(l[, c('Evil_weight', 'Amused_weight', 'Polite_weight')])
plot(l_dir)
mod_l = DirichReg(l_dir ~ cond, l)
newdata = data.frame(cond = levels(l$cond))
pred_l = DR_data(predict(mod_l, newdata))
rownames(pred_l) = levels(l$cond)
colnames(pred_l) = c('Evil_weight', 'Amused_weight', 'Polite_weight')
plot(pred_l)
pred_l


# moans
mgg = read.csv('mgg.csv')[, -1]
# colnames(mgg)

boxplot(Effortful_weight ~ cond, mgg)
boxplot(Hurt_weight ~ cond, mgg)
boxplot(Pleased_weight ~ cond, mgg)

mgg_dir = DR_data(mgg[, c('Hurt_weight', 'Pleased_weight', 'Effortful_weight')])
plot(mgg_dir)
mod_mgg = DirichReg(mgg_dir ~ cond, mgg)
newdata = data.frame(cond = levels(mgg$cond))
pred_mgg = DR_data(predict(mod_mgg, newdata))
rownames(pred_mgg) = levels(mgg$cond)
colnames(pred_mgg) = c('Hurt_weight', 'Pleased_weight', 'Effortful_weight')
plot(pred_mgg)
pred_mgg


# screams
sr = read.csv('sr.csv')[, -1]
# colnames(sr)

boxplot(Aggressive_weight ~ cond, sr)
boxplot(Afraid_weight ~ cond, sr)
boxplot(Pleased_weight ~ cond, sr)

sr_dir = DR_data(sr[, c('Afraid_weight', 'Aggressive_weight', 'Pleased_weight')])
plot(sr_dir)
mod_sr = DirichReg(sr_dir ~ cond, sr)
newdata = data.frame(cond = levels(sr$cond))
pred_sr = DR_data(predict(mod_sr, newdata))
rownames(pred_sr) = levels(sr$cond)
colnames(pred_sr) = c('Afraid_weight', 'Aggressive_weight', 'Pleased_weight')
plot(pred_sr)
pred_sr

mod_srr = DirichReg(sr_dir ~ rolloff, sr)
newdata = data.frame(rolloff = levels(sr$rolloff))
pred_srr = DR_data(predict(mod_srr, newdata))
rownames(pred_srr) = levels(sr$rolloff)
colnames(pred_srr) = c('Afraid_weight', 'Aggressive_weight', 'Pleased_weight')
plot(pred_srr)
pred_srr

## Trying Dirichlet regression
library(DirichletReg)  # see https://cran.r-project.org/web/packages/DirichletReg/vignettes/DirichletReg-vig.pdf   and http://r-statistics.co/Dirichlet-Regression-With-R.html 

# head(ArcticLake)
# AL <- DR_data(ArcticLake[, 1:3])
# plot(AL)
# lake1 <- DirichReg(AL ~ depth, ArcticLake)
# summary(lake1)

l_dir = DR_data(l[, c('Evil..jeering._weight', 'Amused_weight', 'Polite_weight')])
plot(l_dir)
mod_l = DirichReg(l_dir ~ nonlin, l)
summary(mod_l)

newdata = data.frame(nonlin = levels(l$nonlin))
pred_l = DR_data(predict(mod_l, newdata))
rownames(pred_l) = levels(l$nonlin)
colnames(pred_l) = c('Evil..jeering._weight', 'Amused_weight', 'Polite_weight')
plot(pred_l)
pred_l


