## Plotting raw responses as points in the triangle

setwd('/home/allgoodguys/Documents/Studying/Lund_PhD/epistles/014_rolloff/soundgen_confirm')

library(ggplot2)
library(brms)

# df = read.csv('data/rc.csv')[, -1]
gs = read.csv('data/gs.csv')[, -1]
l = read.csv('data/l.csv')[, -1]
mgg = read.csv('data/mgg.csv')[, -1]
sr = read.csv('data/sr.csv')[, -1]

# exclude nonlin conditions
conds = c('O', 'R-', 'R+')
gs = droplevels(gs[gs$cond %in% conds, ])
l = droplevels(l[l$cond %in% conds, ])
mgg = droplevels(mgg[mgg$cond %in% conds, ])
sr = droplevels(sr[sr$cond %in% conds, ])

vertices = list(l1 = data.frame(x = 0, y = 0), 
                l2 = data.frame(x = 200, y = 341),
                l3 = data.frame(x = 400, y = 0))
labels_canonical = list(
  gs = c('Hurt', 'Surprised', 'Pleased'),
  l = c('Evil', 'Polite', 'Amused'),
  mgg = c('Hurt', 'Effortful', 'Pleased'),
  sr = c('Afraid', 'Aggressive', 'Pleased')
)
cond_labels = list('O' = 'O',  # condition labels on the plot
                   'R-' = 'R-',
                   'R+' = 'R+')
full_names = list(gs = 'Gasps', 
                  l = 'Laughs', 
                  mgg = 'Moans', 
                  sr = 'Screams')
titles = paste(c('A.', 'B.', 'C.', 'D.'), full_names)
mycol = list(
  'O' = 'gray50',
  'R-' = 'gray60',
  'R+' = 'gray40'
)
mycol_rgb = lapply(mycol, col2rgb, alpha = TRUE)
mypch = list(
  'O' = 16,
  'R-' = 18,
  'R+' = 17
)

op = par()
tiff(filename = 'pix/fig_observed_arrows.tiff', width = 16, height = 15, units = 'cm', res = 1200, compression = 'lzw')
par(mfrow = c(2, 2),     # 2x2 layout
    oma = c(0, 0, 0, 1), # rows of text at margins: bottom-left-top-right
    mar = c(1, 1, 0, 0), # space for one row of text at ticks and to separate plots
    xpd = NA)            # allow content to protrude into outer margin (and beyond)
for (c in 1:length(names(labels_canonical))) {
  n = names(labels_canonical)[c]
  plot(1:2, 1:2, type = 'n', 
       xlim = c(-20, 420), ylim = c(-10, 380), 
       axes = FALSE, mar = c(0, 0, 4, 0),
       xlab = '', ylab = '')  # main = full_names[[c]], 
  
  temp = get(n)
  temp$r = temp$g = temp$b = temp$alpha = NA
  
  # aggregate per sound and cond
  pos_agg1 = aggregate(coordX_st ~ cond + sound_base, temp, mean)
  pos_agg1$coordY_st = aggregate(coordY_st ~ cond + sound_base, temp, mean)[, 3]
  pos_agg1$r = pos_agg1$g = pos_agg1$b = pos_agg1$alpha = NA
  for (i in 1:nrow(pos_agg1)) {
    pos_agg1[i, c('r', 'g', 'b', 'alpha')] = unlist(mycol_rgb[as.character(pos_agg1$cond[i])])
    pos_agg1$pch[i] = unlist(mypch[as.character(pos_agg1$cond[i])])
  }
  points(pos_agg1$coordX_st, 
         pos_agg1$coordY_st, 
         pch = pos_agg1$pch, 
         cex = 1.2, 
         col = rgb(red = pos_agg1$r, green = pos_agg1$g, blue = pos_agg1$b, alpha = 255/2, maxColorValue = 255))  # col = rgb(0, 0, 0, alpha = .1))
  
  # add segments joining O with other conditions
  for (s in unique(pos_agg1$sound_base)) {
    mySound = pos_agg1[pos_agg1$sound_base == as.character(s), ]
    conds = unique(mySound$cond)
    conds = as.character(conds[conds != 'O'])
    for (myCond in conds) {
      arrows(x0 = mySound$coordX_st[mySound$cond == 'O'], 
             y0 = mySound$coordY_st[mySound$cond == 'O'],
             x1 = mySound$coordX_st[mySound$cond == myCond], 
             y1 = mySound$coordY_st[mySound$cond == myCond],
             length = .075, angle = 15, lwd = .4,
             col = rgb(0, 0, 0, alpha = 200, maxColorValue = 255))
    }
  }
  
  # draw triangle
  segments(x0 = vertices[[1]]$x, y0 = vertices[[1]]$y,
           x1 = vertices[[2]]$x, y1 = vertices[[2]]$y)
  segments(x0 = vertices[[1]]$x, y0 = vertices[[1]]$y,
           x1 = vertices[[3]]$x, y1 = vertices[[3]]$y)
  segments(x0 = vertices[[2]]$x, y0 = vertices[[2]]$y,
           x1 = vertices[[3]]$x, y1 = vertices[[3]]$y)
  
  # label vertices
  text(vertices[[1]]$x, vertices[[1]]$y - 10, labels = labels_canonical[[n]][1])
  text(vertices[[2]]$x, vertices[[2]]$y + 10, labels = labels_canonical[[n]][2])
  text(vertices[[3]]$x, vertices[[3]]$y - 10, labels = labels_canonical[[n]][3])
  text(-10, 380, labels = titles[c], pos = 4, cex = 1.3)
  
  # add aggregated coordinates per condition
  pos_agg = aggregate(coordX_st ~ cond, temp, mean)
  pos_agg$coordY_st = aggregate(coordY_st ~ cond, temp, mean)[, 2]
  text(pos_agg$coordX_st, 
       pos_agg$coordY_st, 
       labels = unlist(cond_labels[which(names(cond_labels) == as.character(pos_agg$cond))]), 
       cex = .9, font = 2)
  
  # legend
  if (c == 1) {
    legend("right",
           legend = names(cond_labels),
           pch = unlist(mypch),
           col = unlist(mycol),
           bty = 'n')
  }
}
dev.off()
par(op)