## Modeling weights with softmax transform

setwd('/home/allgoodguys/Documents/Studying/Lund_PhD/epistles/014_rolloff/soundgen_confirm')

library(ggplot2)
library(rstan)

fullSample = c(TRUE, FALSE)[2]  # include all participants or remove the "noisy" data?
if (fullSample) {
  # df = read.csv('data/rc_full.csv')[, -1]
  gs = read.csv('data/gs_full.csv')[, -1]
  l = read.csv('data/l_full.csv')[, -1]
  mgg = read.csv('data/mgg_full.csv')[, -1]
  sr = read.csv('data/sr_full.csv')[, -1]
} else {
  # df = read.csv('data/rc.csv')[, -1]
  gs = read.csv('data/gs.csv')[, -1]
  l = read.csv('data/l.csv')[, -1]
  mgg = read.csv('data/mgg.csv')[, -1]
  sr = read.csv('data/sr.csv')[, -1]
}


# exclude nonlin conditions
conds = c('O', 'R-', 'R+')
gs = droplevels(gs[gs$cond %in% conds, ])
l = droplevels(l[l$cond %in% conds, ])
mgg = droplevels(mgg[mgg$cond %in% conds, ])
sr = droplevels(sr[sr$cond %in% conds, ])


vertices = list(l1 = data.frame(x = 0, y = 0), 
                l2 = data.frame(x = 200, y = 341),
                l3 = data.frame(x = 400, y = 0))
labels_canonical = list(
  gs = c('Hurt', 'Surprised', 'Pleased'),
  l = c('Evil', 'Polite', 'Amused'),
  mgg = c('Hurt', 'Effortful', 'Pleased'),
  sr = c('Afraid', 'Aggressive', 'Pleased')
)
full_names = list(gs = 'Gasps', 
                  l = 'Laughs', 
                  mgg = 'Moans', 
                  sr = 'Screams')
weights = list(
  gs = c('Hurt_weight', 'Pleased_weight', 'Surprised_weight'),
  l = c('Amused_weight', 'Polite_weight', 'Evil_weight'),
  mgg = c('Effortful_weight', 'Hurt_weight', 'Pleased_weight'),
  sr = c('Afraid_weight', 'Aggressive_weight', 'Pleased_weight')
)


## basic STAN model: "poor-man's Dirichlet" (softmax-based)
modelString = "
data {
  int<lower=1> N;  // number of trials
  int<lower=1> K;  // number of levels of cond (Ch, O, etc)
  vector[N] w1;    // weight of the first label in the triangle (Hurt_weight etc)
  vector[N] w2;
  int cond[N];  // cond (Ch, O, etc)
  int id[N];    // participant id
  int<lower=1> n_id;  // number of unique IDs
  int sound_base[N];    // sound
  int<lower=1> n_sound_base;  // number of unique sounds
}

parameters {
  real phi1[K];
  real phi2[K];
  real phi3[K];
  real<lower=0, upper=1> sigma;
  real<lower = -1, upper=1> b1_id[n_id];
  real<lower = -1, upper=1> b2_id[n_id];
  real<lower = -1, upper=1> b1_sound_base[n_sound_base];
  real<lower = -1, upper=1> b2_sound_base[n_sound_base];
  real<lower = 0, upper=1> sigma_id;
  real<lower = 0, upper=1> sigma_sound_base;
}

transformed parameters {
  real mu1[K];
  real mu2[K];
  real mu3[K];
  for (c in 1:K) {
    mu1[c] = exp(phi1[c]) / ( exp(phi1[c]) + exp(phi2[c]) + exp(phi3[c]) );
    mu2[c] = exp(phi2[c]) / ( exp(phi1[c]) + exp(phi2[c]) + exp(phi3[c]) );
    mu3[c] = 1 - mu1[c] - mu2[c];
  }
}

model {
  sigma ~ cauchy(0, .5);
  sigma_id ~ cauchy(0, .5);
  sigma_sound_base ~ cauchy(0, .5);
  b1_id ~ normal(0, sigma_id);
  b2_id ~ normal(0, sigma_id);
  b1_sound_base ~ normal(0, sigma_sound_base);
  b2_sound_base ~ normal(0, sigma_sound_base);
  for (c in 1:K) {
    phi1[c] ~ normal(0, 1);
    phi2[c] ~ normal(0, 1);
    phi3[c] ~ normal(0, 1);
  }
  for (i in 1:N) {
    w1[i] ~ normal(mu1[cond[i]] + b1_id[id[i]] + b1_sound_base[sound_base[i]], sigma);
    w2[i] ~ normal(mu2[cond[i]] + b2_id[id[i]] + b2_sound_base[sound_base[i]], sigma);
  }
}
"
stanmod = stan_model(model_code = modelString, model_name = "stanmod") 
# NB: mu1[1:5] gives the weight of vertex 1[conditions 1 to 5], mu2 same for vertex 2, mu3 for vertex 3


##
## GASPS 
## 
# apply(gs[, c('Hurt_weight', 'Pleased_weight', 'Surprised_weight')], 1, sum)  # sum to 100

# simple plotting
gs1 = reshape(gs[, c('cond', 'Hurt_weight', 'Pleased_weight', 'Surprised_weight')],
              varying = c('Hurt_weight', 'Pleased_weight', 'Surprised_weight'),
              v.names = 'weight',
              timevar = 'em',
              times = c('Hurt', 'Pleased', 'Surprised'),
              direction = 'long')
summary(gs1)
boxplot(weight ~ cond + em, gs1)

dataList_gs = list(
  N = nrow(gs),
  K = length(unique(gs$cond)),
  w1 = gs[, weights$gs[1]] / 100,
  w2 = gs[, weights$gs[2]] / 100,
  cond = as.numeric(gs$cond),   # df$cond[1:10]; as.numeric(df$cond[1:10])
  id = as.numeric(gs$id),
  n_id = length(unique(gs$id)),
  sound_base = as.numeric(as.factor(gs$sound_base)),
  n_sound_base = length(unique(gs$sound_base))
)

# fit_gs = sampling(stanmod, data = dataList_gs, chains = 2, cores = 2, iter = 4000, thin = 1, pars = NA)  # save all
# saveRDS(fit_gs, 'models/fit_gs.RDS')  # saveRDS(fit_gs, 'models/fit_gs_full.RDS')
fit_gs = readRDS('models/fit_gs.RDS')   # fit_gs = readRDS('models/fit_gs_full.RDS') 
# stan_trace(fit_gs, pars = c('sigma', 'sigma_id', 'sigma_sound_base', 'mu1'))
coda_gs = as.matrix(fit_gs)
# colnames(coda_gs)
coda_gs = coda_gs[, which(substr(colnames(coda_gs), 1, 2) == 'mu')] * 100  # just the mu's
colnames(coda_gs) = paste0(rep(weights$gs, each = length(unique(gs$cond))), '_', rep(levels(gs$cond), 3))
# summary(coda_gs)
coda_gs = as.data.frame(coda_gs)

# contrasts
comparisons = data.frame(Contrast = c('R+ vs. R-', 'R+ vs. O', 'O vs. R-'))
for (i in 1:3) {
  j = length(unique(gs$cond)) * (i - 1) + 1
  temp = coda_gs[, j:(j+length(unique(gs$cond))-1)]
  colnames(temp) = levels(gs$cond)
  comparisons[which(comparisons$Contrast=='R+ vs. R-'), weights$gs[i]] = soundgen:::reportCI(quantile(temp[,'R+'] - temp[,'R-'], probs = c(.5, .025, .975)), digits = 1)
  comparisons[which(comparisons$Contrast=='R+ vs. O'), weights$gs[i]] = soundgen:::reportCI(quantile(temp[,'R+'] - temp[,'O'], probs = c(.5, .025, .975)), digits = 1)
  comparisons[which(comparisons$Contrast=='O vs. R-'), weights$gs[i]] = soundgen:::reportCI(quantile(temp[,'O'] - temp[,'R-'], probs = c(.5, .025, .975)), digits = 1)
}
comparisons = comparisons[, c('Contrast', 'Hurt_weight', 'Surprised_weight', 'Pleased_weight')]
write.csv(comparisons, '~/Downloads/gs.csv')

# sanity check: 
aggregate(get(weights$gs[1])~cond, gs, mean)
aggregate(get(weights$gs[2])~cond, gs, mean)
aggregate(get(weights$gs[3])~cond, gs, mean)


##
## LAUGHS 
## 
dataList_l = list(
  N = nrow(l),
  K = length(unique(l$cond)),
  w1 = l[, weights$l[1]] / 100,
  w2 = l[, weights$l[2]] / 100,
  cond = as.numeric(l$cond),   # df$cond[1:10]; as.numeric(df$cond[1:10])
  id = as.numeric(l$id),
  n_id = length(unique(l$id)),
  sound_base = as.numeric(as.factor(l$sound_base)),
  n_sound_base = length(unique(l$sound_base))
)

# fit_l = sampling(stanmod, data = dataList_l, chains = 2, cores = 2, iter = 4000, thin = 1, pars = NA)  # save all
# saveRDS(fit_l, 'models/fit_l.RDS')  # saveRDS(fit_l, 'models/fit_l_full.RDS')
fit_l = readRDS('models/fit_l.RDS')  # fit_l = readRDS('models/fit_l_full.RDS')
# stan_trace(fit_l, pars = c('sigma', 'sigma_id', 'sigma_sound_base', 'mu1'))
coda_l = as.matrix(fit_l)
# colnames(coda_l)
coda_l = coda_l[, which(substr(colnames(coda_l), 1, 2) == 'mu')] * 100  # just the mu's
colnames(coda_l) = paste0(rep(weights$l, each = length(unique(l$cond))), '_', rep(levels(l$cond), 3))
# summary(coda_l)
coda_l = as.data.frame(coda_l)

# contrasts
comparisons = data.frame(Contrast = c('R+ vs. R-', 'R+ vs. O', 'O vs. R-'))
for (i in 1:3) {
  j = length(unique(l$cond)) * (i - 1) + 1
  temp = coda_l[, j:(j+length(unique(l$cond))-1)]
  colnames(temp) = levels(l$cond)
  comparisons[which(comparisons$Contrast=='R+ vs. R-'), weights$l[i]] = soundgen:::reportCI(quantile(temp[,'R+'] - temp[,'R-'], probs = c(.5, .025, .975)), digits = 1)
  comparisons[which(comparisons$Contrast=='R+ vs. O'), weights$l[i]] = soundgen:::reportCI(quantile(temp[,'R+'] - temp[,'O'], probs = c(.5, .025, .975)), digits = 1)
  comparisons[which(comparisons$Contrast=='O vs. R-'), weights$l[i]] = soundgen:::reportCI(quantile(temp[,'O'] - temp[,'R-'], probs = c(.5, .025, .975)), digits = 1)
}
comparisons = comparisons[, c('Contrast', 'Evil_weight', 'Polite_weight', 'Amused_weight')]
write.csv(comparisons, '~/Downloads/laugh.csv')

# sanity check: 
aggregate(get(weights$l[1])~cond, l, mean)
aggregate(get(weights$l[2])~cond, l, mean)
aggregate(get(weights$l[3])~cond, l, mean)


##
## MOANS 
## 
dataList_mgg = list(
  N = nrow(mgg),
  K = length(unique(mgg$cond)),
  w1 = mgg[, weights$mgg[1]] / 100,
  w2 = mgg[, weights$mgg[2]] / 100,
  cond = as.numeric(mgg$cond),   # df$cond[1:10]; as.numeric(df$cond[1:10])
  id = as.numeric(mgg$id),
  n_id = length(unique(mgg$id)),
  sound_base = as.numeric(as.factor(mgg$sound_base)),
  n_sound_base = length(unique(mgg$sound_base))
)

# fit_mgg = sampling(stanmod, data = dataList_mgg, chains = 2, cores = 2, iter = 4000, thin = 1, pars = NA)  # save all
# saveRDS(fit_mgg, 'models/fit_mgg.RDS')  # saveRDS(fit_mgg, 'models/fit_mgg_full.RDS')
fit_mgg = readRDS('models/fit_mgg.RDS')  # fit_mgg = readRDS('models/fit_mgg_full.RDS')
# stan_trace(fit_mgg, pars = c('sigma', 'sigma_id', 'sigma_sound_base', 'mu1'))
coda_mgg = as.matrix(fit_mgg)
# colnames(coda_mgg)
coda_mgg = coda_mgg[, which(substr(colnames(coda_mgg), 1, 2) == 'mu')] * 100  # just the mu's
colnames(coda_mgg) = paste0(rep(weights$mgg, each = length(unique(mgg$cond))), '_', rep(levels(mgg$cond), 3))
# summary(coda_mgg)
coda_mgg = as.data.frame(coda_mgg)

# contrasts
comparisons = data.frame(Contrast = c('R+ vs. R-', 'R+ vs. O', 'O vs. R-'))
for (i in 1:3) {
  j = length(unique(mgg$cond)) * (i - 1) + 1
  temp = coda_mgg[, j:(j+length(unique(mgg$cond))-1)]
  colnames(temp) = levels(mgg$cond)
  comparisons[which(comparisons$Contrast=='R+ vs. R-'), weights$mgg[i]] = soundgen:::reportCI(quantile(temp[,'R+'] - temp[,'R-'], probs = c(.5, .025, .975)), digits = 1)
  comparisons[which(comparisons$Contrast=='R+ vs. O'), weights$mgg[i]] = soundgen:::reportCI(quantile(temp[,'R+'] - temp[,'O'], probs = c(.5, .025, .975)), digits = 1)
  comparisons[which(comparisons$Contrast=='O vs. R-'), weights$mgg[i]] = soundgen:::reportCI(quantile(temp[,'O'] - temp[,'R-'], probs = c(.5, .025, .975)), digits = 1)
}
comparisons = comparisons[, c('Contrast', 'Hurt_weight', 'Effortful_weight', 'Pleased_weight')]
write.csv(comparisons, '~/Downloads/mgg.csv')

# sanity check: 
aggregate(get(weights$mgg[1])~cond, mgg, mean)
aggregate(get(weights$mgg[2])~cond, mgg, mean)
aggregate(get(weights$mgg[3])~cond, mgg, mean)


##
## SCREAMS 
## 
# apply(sr[, c('Hurt_weight', 'Pleased_weight', 'Surprised_weight')], 1, sum)  # sum to 100
dataList_sr = list(
  N = nrow(sr),
  K = length(unique(sr$cond)),
  w1 = sr[, weights$sr[1]] / 100,
  w2 = sr[, weights$sr[2]] / 100,
  cond = as.numeric(sr$cond),   # df$cond[1:10]; as.numeric(df$cond[1:10])
  id = as.numeric(sr$id),
  n_id = length(unique(sr$id)),
  sound_base = as.numeric(as.factor(sr$sound_base)),
  n_sound_base = length(unique(sr$sound_base))
)

# fit_sr = sampling(stanmod, data = dataList_sr, chains = 2, cores = 2, iter = 4000, thin = 1, pars = NA)  # save all
# saveRDS(fit_sr, 'models/fit_sr.RDS')  # saveRDS(fit_sr, 'models/fit_sr_full.RDS')
fit_sr = readRDS('models/fit_sr.RDS')  # fit_sr = readRDS('models/fit_sr_full.RDS')
# stan_trace(fit_sr, pars = c('sigma', 'sigma_id', 'sigma_sound_base', 'mu1'))
coda_sr = as.matrix(fit_sr)
# colnames(coda_sr)
coda_sr = coda_sr[, which(substr(colnames(coda_sr), 1, 2) == 'mu')] * 100  # just the mu's
colnames(coda_sr) = paste0(rep(weights$sr, each = length(unique(sr$cond))), '_', rep(levels(sr$cond), 3))
# summary(coda_sr)
coda_sr = as.data.frame(coda_sr)

# contrasts
comparisons = data.frame(Contrast = c('R+ vs. R-', 'R+ vs. O', 'O vs. R-'))
for (i in 1:3) {
  j = length(unique(sr$cond)) * (i - 1) + 1
  temp = coda_sr[, j:(j+length(unique(sr$cond))-1)]
  colnames(temp) = levels(sr$cond)
  comparisons[which(comparisons$Contrast=='R+ vs. R-'), weights$sr[i]] = soundgen:::reportCI(quantile(temp[,'R+'] - temp[,'R-'], probs = c(.5, .025, .975)), digits = 1)
  comparisons[which(comparisons$Contrast=='R+ vs. O'), weights$sr[i]] = soundgen:::reportCI(quantile(temp[,'R+'] - temp[,'O'], probs = c(.5, .025, .975)), digits = 1)
  comparisons[which(comparisons$Contrast=='O vs. R-'), weights$sr[i]] = soundgen:::reportCI(quantile(temp[,'O'] - temp[,'R-'], probs = c(.5, .025, .975)), digits = 1)
}
comparisons = comparisons[, c('Contrast', 'Afraid_weight', 'Aggressive_weight', 'Pleased_weight')]
write.csv(comparisons, '~/Downloads/sr.csv')

# sanity check: 
aggregate(get(weights$sr[1])~cond, sr, mean)
aggregate(get(weights$sr[2])~cond, sr, mean)
aggregate(get(weights$sr[3])~cond, sr, mean)

