## Modeling weights with softmax transform CONTROLLING FOR spectral centroid

setwd('/home/allgoodguys/Documents/Studying/Lund_PhD/epistles/014_rolloff/soundgen_confirm')

library(ggplot2)
library(rstan)

fullSample = c(TRUE, FALSE)[2]  # include all participants or remove the "noisy" data?
if (fullSample) {
  # df = read.csv('data/rc_full.csv')[, -1]
  gs = read.csv('data/gs_full.csv')[, -1]
  l = read.csv('data/l_full.csv')[, -1]
  mgg = read.csv('data/mgg_full.csv')[, -1]
  sr = read.csv('data/sr_full.csv')[, -1]
} else {
  # df = read.csv('data/rc.csv')[, -1]
  gs = read.csv('data/gs.csv')[, -1]
  l = read.csv('data/l.csv')[, -1]
  mgg = read.csv('data/mgg.csv')[, -1]
  sr = read.csv('data/sr.csv')[, -1]
}

# exclude nonlin conditions
conds = c('O', 'R-', 'R+')
df = droplevels(df[df$cond %in% conds, ])
gs = droplevels(gs[gs$cond %in% conds, ])
l = droplevels(l[l$cond %in% conds, ])
mgg = droplevels(mgg[mgg$cond %in% conds, ])
sr = droplevels(sr[sr$cond %in% conds, ])

vertices = list(l1 = data.frame(x = 0, y = 0), 
                l2 = data.frame(x = 200, y = 341),
                l3 = data.frame(x = 400, y = 0))
labels_canonical = list(
  gs = c('Hurt', 'Surprised', 'Pleased'),
  l = c('Evil', 'Polite', 'Amused'),
  mgg = c('Hurt', 'Effortful', 'Pleased'),
  sr = c('Afraid', 'Aggressive', 'Pleased')
)
full_names = list(gs = 'Gasps', 
                  l = 'Laughs', 
                  mgg = 'Moans', 
                  sr = 'Screams')
weights = list(
  gs = c('Hurt_weight', 'Pleased_weight', 'Surprised_weight'),
  l = c('Amused_weight', 'Polite_weight', 'Evil_weight'),
  mgg = c('Effortful_weight', 'Hurt_weight', 'Pleased_weight'),
  sr = c('Afraid_weight', 'Aggressive_weight', 'Pleased_weight')
)


## basic STAN model: "poor-man's Dirichlet" (softmax-based)
modelString = "
  data {
    int<lower=1> N;  // number of trials
    int<lower=1> K;  // number of levels of cond (Ch, O, etc)
    vector[N] w1;    // weight of the first label in the triangle (Hurt_weight etc)
    vector[N] w2;
    int cond[N];  // cond (Ch, O, etc)
    int id[N];    // participant id
    int<lower=1> n_id;  // number of unique IDs
    int sound_base[N];    // sound
    int<lower=1> n_sound_base;  // number of unique sounds
    vector[N] specCentroid;
  }

parameters {
  real phi1[K];
  real phi2[K];
  real phi3[K];
  real<lower=0, upper=1> sigma;
  real<lower = -1, upper=1> b1_id[n_id];
  real<lower = -1, upper=1> b2_id[n_id];
  real<lower = -1, upper=1> b1_sound_base[n_sound_base];
  real<lower = -1, upper=1> b2_sound_base[n_sound_base];
  real<lower = 0, upper=1> sigma_id;
  real<lower = 0, upper=1> sigma_sound_base;
  real<lower = -1, upper=1> b1_specCentroid;
  real<lower = -1, upper=1> b2_specCentroid;
}

transformed parameters {
  real mu1[K];
  real mu2[K];
  real mu3[K];
  for (c in 1:K) {
    mu1[c] = exp(phi1[c]) / ( exp(phi1[c]) + exp(phi2[c]) + exp(phi3[c]) );
    mu2[c] = exp(phi2[c]) / ( exp(phi1[c]) + exp(phi2[c]) + exp(phi3[c]) );
    mu3[c] = 1 - mu1[c] - mu2[c];
  }
}

model {
  // priors
  sigma ~ cauchy(0, .5);
  sigma_id ~ cauchy(0, .5);
  sigma_sound_base ~ cauchy(0, .5);
  b1_id ~ normal(0, sigma_id);
  b2_id ~ normal(0, sigma_id);
  b1_sound_base ~ normal(0, sigma_sound_base);
  b2_sound_base ~ normal(0, sigma_sound_base);
  b1_specCentroid ~ normal(0, 1);
  b2_specCentroid ~ normal(0, 1);

  // likelihood
  for (c in 1:K) {
    phi1[c] ~ normal(0, 1);
    phi2[c] ~ normal(0, 1);
    phi3[c] ~ normal(0, 1);
  }
  for (i in 1:N) {
    w1[i] ~ normal(mu1[cond[i]] + b1_specCentroid * specCentroid[i] + b1_id[id[i]] + b1_sound_base[sound_base[i]], sigma);
    w2[i] ~ normal(mu2[cond[i]] + b2_specCentroid * specCentroid[i] + b2_id[id[i]] + b2_sound_base[sound_base[i]], sigma);
  }
}
"
stanmod = stan_model(model_code = modelString, model_name = "stanmod") 
# NB: mu1[1:5] gives the weight of vertex 1[conditions 1 to 5], mu2 same for vertex 2, mu3 for vertex 3


##
## GASPS 
## 
gs$log_sc = log10(gs$specCentroid_median)

dataList_gs = list(
  N = nrow(gs),
  K = length(unique(gs$cond)),
  w1 = gs[, weights$gs[1]] / 100,
  w2 = gs[, weights$gs[2]] / 100,
  cond = as.numeric(gs$cond),   # df$cond[1:10]; as.numeric(df$cond[1:10])
  id = as.numeric(gs$id),
  n_id = length(unique(gs$id)),
  sound_base = as.numeric(as.factor(gs$sound_base)),
  n_sound_base = length(unique(gs$sound_base)),
  specCentroid = as.vector(gs$log_sc)
)

# fit_gs_extraPred = sampling(stanmod, data = dataList_gs, pars = NA, iter = 5000, warmup = 1000, thin = 1, chains = 2, cores = 2)
# saveRDS(fit_gs_extraPred, 'models/fit_gs_extraPred.RDS')  # saveRDS(fit_gs_extraPred, 'models/fit_gs_extraPred_full.RDS')
fit_gs_extraPred = readRDS('models/fit_gs_extraPred.RDS')  # fit_gs_extraPred = readRDS('models/fit_gs_extraPred_full.RDS')
# stan_trace(fit_gs_extraPred, pars = c('sigma', 'sigma_id', 'sigma_sound_base', 'b1_specCentroid', 'b2_specCentroid', 'mu1'))
coda_gs_full = as.matrix(fit_gs_extraPred)
# colnames(coda_gs_full)
coda_gs_full = as.data.frame(coda_gs_full)
coda_gs_full$b3_specCentroid = -(coda_gs_full$b1_specCentroid + coda_gs_full$b2_specCentroid)  # simplex, so changes have to sum to 0

coda_gs = coda_gs_full[, which(substr(colnames(coda_gs_full), 1, 2) == 'mu')] * 100  # just the mu's
colnames(coda_gs) = paste0(rep(weights$gs, each = length(unique(gs$cond))), '_', rep(levels(gs$cond), 3))
coda_gs = cbind(coda_gs, coda_gs_full[, c('b1_specCentroid', 'b2_specCentroid', 'b3_specCentroid')])
# summary(coda_gs)

# contrasts
comparisons = data.frame(Contrast = c('R+ vs. R-', 'Spectral centroid'))
for (i in 1:3) {
  j = length(unique(gs$cond)) * (i - 1) + 1
  temp = coda_gs[, j:(j+length(unique(gs$cond))-1)]
  colnames(temp) = levels(gs$cond)
  comparisons[which(comparisons$Contrast=='R+ vs. R-'), weights$gs[i]] = soundgen:::reportCI(quantile(temp[,'R+'] - temp[,'R-'], probs = c(.5, .025, .975)), digits = 1)
  bName = paste0('b', i, '_specCentroid')
  comparisons[which(comparisons$Contrast=='Spectral centroid'), weights$gs[i]] = soundgen:::reportCI(quantile(coda_gs[, bName] * diff(range(gs$log_sc)) * 100, probs = c(.5, .025, .975)), digits = 1)
}
comparisons = comparisons[, c('Contrast', 'Hurt_weight', 'Surprised_weight', 'Pleased_weight')]
write.csv(comparisons, '~/Downloads/temp_gs_extraPred.csv')



##
## LAUGHS 
## 
l$log_sc = log(l$specCentroid_median)

dataList_l = list(
  N = nrow(l),
  K = length(unique(l$cond)),
  w1 = l[, weights$l[1]] / 100,
  w2 = l[, weights$l[2]] / 100,
  cond = as.numeric(l$cond),   # df$cond[1:10]; as.numeric(df$cond[1:10])
  id = as.numeric(l$id),
  n_id = length(unique(l$id)),
  sound_base = as.numeric(as.factor(l$sound_base)),
  n_sound_base = length(unique(l$sound_base)),
  specCentroid = as.vector(l$log_sc)
)

# fit_l = sampling(stanmod, data = dataList_l, pars = NA, iter = 5000, warmup = 1000, thin = 1, chains = 2, cores = 2)
# saveRDS(fit_l, 'models/fit_l_extraPred.RDS')  # saveRDS(fit_l, 'models/fit_l_extraPred_full.RDS')
fit_l = readRDS('models/fit_l_extraPred.RDS')  # fit_l = readRDS('models/fit_l_extraPred_full.RDS')
# stan_trace(fit_l, pars = c('sigma', 'sigma_id', 'sigma_sound_base', 'b1_specCentroid', 'b2_specCentroid', 'mu1'))
coda_l_full = as.matrix(fit_l)
# colnames(coda_l_full)
coda_l_full = as.data.frame(coda_l_full)
coda_l_full$b3_specCentroid = -(coda_l_full$b1_specCentroid + coda_l_full$b2_specCentroid)  # simplex, so changes have to sum to 0

coda_l = coda_l_full[, which(substr(colnames(coda_l_full), 1, 2) == 'mu')] * 100  # just the mu's
colnames(coda_l) = paste0(rep(weights$l, each = length(unique(l$cond))), '_', rep(levels(l$cond), 3))
coda_l = cbind(coda_l, coda_l_full[, c('b1_specCentroid', 'b2_specCentroid', 'b3_specCentroid')])
# summary(coda_l)

# contrasts
comparisons = data.frame(Contrast = c('R+ vs. R-', 'Spectral centroid'))
for (i in 1:3) {
  j = length(unique(l$cond)) * (i - 1) + 1
  temp = coda_l[, j:(j+length(unique(l$cond))-1)]
  colnames(temp) = levels(l$cond)
  comparisons[which(comparisons$Contrast=='R+ vs. R-'), weights$l[i]] = soundgen:::reportCI(quantile(temp[,'R+'] - temp[,'R-'], probs = c(.5, .025, .975)), digits = 1)
  bName = paste0('b', i, '_specCentroid')
  comparisons[which(comparisons$Contrast=='Spectral centroid'), weights$l[i]] = soundgen:::reportCI(quantile(coda_l[, bName] * diff(range(l$log_sc)) * 100, probs = c(.5, .025, .975)), digits = 1)
}
comparisons = comparisons[, c('Contrast', 'Evil_weight', 'Polite_weight', 'Amused_weight')]
write.csv(comparisons, '~/Downloads/temp_l_extraPred.csv')



##
## MOANS 
## 
mgg$log_sc = log(mgg$specCentroid_median)

dataList_mgg = list(
  N = nrow(mgg),
  K = length(unique(mgg$cond)),
  w1 = mgg[, weights$mgg[1]] / 100,
  w2 = mgg[, weights$mgg[2]] / 100,
  cond = as.numeric(mgg$cond),   # df$cond[1:10]; as.numeric(df$cond[1:10])
  id = as.numeric(mgg$id),
  n_id = length(unique(mgg$id)),
  sound_base = as.numeric(as.factor(mgg$sound_base)),
  n_sound_base = length(unique(mgg$sound_base)),
  specCentroid = as.vector(mgg$log_sc)
)

# fit_mgg = sampling(stanmod, data = dataList_mgg, pars = NA, iter = 5000, warmup = 1000, thin = 1, chains = 2, cores = 2)
# saveRDS(fit_mgg, 'models/fit_mgg_extraPred.RDS')  # saveRDS(fit_mgg, 'models/fit_mgg_extraPred_full.RDS')
fit_mgg = readRDS('models/fit_mgg_extraPred.RDS')  # fit_mgg = readRDS('models/fit_mgg_extraPred_full.RDS')
# stan_trace(fit_mgg, pars = c('sigma', 'sigma_id', 'sigma_sound_base', 'b1_specCentroid', 'b2_specCentroid', 'mu1'))
coda_mgg_full = as.matrix(fit_mgg)
# colnames(coda_mgg_full)
coda_mgg_full = as.data.frame(coda_mgg_full)
coda_mgg_full$b3_specCentroid = -(coda_mgg_full$b1_specCentroid + coda_mgg_full$b2_specCentroid)  # simplex, so changes have to sum to 0

coda_mgg = coda_mgg_full[, which(substr(colnames(coda_mgg_full), 1, 2) == 'mu')] * 100  # just the mu's
colnames(coda_mgg) = paste0(rep(weights$mgg, each = length(unique(mgg$cond))), '_', rep(levels(mgg$cond), 3))
coda_mgg = cbind(coda_mgg, coda_mgg_full[, c('b1_specCentroid', 'b2_specCentroid', 'b3_specCentroid')])
# summary(coda_mgg)

# contrasts
comparisons = data.frame(Contrast = c('R+ vs. R-', 'Spectral centroid'))
for (i in 1:3) {
  j = length(unique(mgg$cond)) * (i - 1) + 1
  temp = coda_mgg[, j:(j+length(unique(mgg$cond))-1)]
  colnames(temp) = levels(mgg$cond)
  comparisons[which(comparisons$Contrast=='R+ vs. R-'), weights$mgg[i]] = soundgen:::reportCI(quantile(temp[,'R+'] - temp[,'R-'], probs = c(.5, .025, .975)), digits = 1)
  bName = paste0('b', i, '_specCentroid')
  comparisons[which(comparisons$Contrast=='Spectral centroid'), weights$mgg[i]] = soundgen:::reportCI(quantile(coda_mgg[, bName] * diff(range(mgg$log_sc)) * 100, probs = c(.5, .025, .975)), digits = 1)
}
comparisons = comparisons[, c('Contrast', 'Hurt_weight', 'Effortful_weight', 'Pleased_weight')]
write.csv(comparisons, '~/Downloads/temp_mgg_extraPred.csv')



##
## SCREAMS 
## 
sr$log_sc = log10(sr$specCentroid_median)

dataList_sr = list(
  N = nrow(sr),
  K = length(unique(sr$cond)),
  w1 = sr[, weights$sr[1]] / 100,
  w2 = sr[, weights$sr[2]] / 100,
  cond = as.numeric(sr$cond),   # df$cond[1:10]; as.numeric(df$cond[1:10])
  id = as.numeric(sr$id),
  n_id = length(unique(sr$id)),
  sound_base = as.numeric(as.factor(sr$sound_base)),
  n_sound_base = length(unique(sr$sound_base)),
  specCentroid = as.vector(sr$specCentroid_median),
  meanFreq = as.vector(sr$specCentroid_median)
)

# fit_sr = sampling(stanmod, data = dataList_sr, pars = NA, iter = 5000, warmup = 1000, thin = 1, chains = 2, cores = 2)
# saveRDS(fit_sr, 'models/fit_sr_extraPred.RDS')  # saveRDS(fit_sr, 'models/fit_sr_extraPred_full.RDS')
fit_sr = readRDS('models/fit_sr_extraPred.RDS')  # fit_sr = readRDS('models/fit_sr_extraPred_full.RDS')
# stan_trace(fit_sr, pars = c('sigma', 'sigma_id', 'sigma_sound_base', 'b1_specCentroid', 'b2_specCentroid', 'mu1'))
coda_sr_full = as.matrix(fit_sr)
# colnames(coda_sr_full)
coda_sr_full = as.data.frame(coda_sr_full)
coda_sr_full$b3_specCentroid = -(coda_sr_full$b1_specCentroid + coda_sr_full$b2_specCentroid)  # simplex, so changes have to sum to 0

coda_sr = coda_sr_full[, which(substr(colnames(coda_sr_full), 1, 2) == 'mu')] * 100  # just the mu's
colnames(coda_sr) = paste0(rep(weights$sr, each = length(unique(sr$cond))), '_', rep(levels(sr$cond), 3))
coda_sr = cbind(coda_sr, coda_sr_full[, c('b1_specCentroid', 'b2_specCentroid', 'b3_specCentroid')])
# summary(coda_sr)

# contrasts
comparisons = data.frame(Contrast = c('R+ vs. R-', 'Spectral centroid'))
for (i in 1:3) {
  j = length(unique(sr$cond)) * (i - 1) + 1
  temp = coda_sr[, j:(j+length(unique(sr$cond))-1)]
  colnames(temp) = levels(sr$cond)
  comparisons[which(comparisons$Contrast=='R+ vs. R-'), weights$sr[i]] = soundgen:::reportCI(quantile(temp[,'R+'] - temp[,'R-'], probs = c(.5, .025, .975)), digits = 1)
  bName = paste0('b', i, '_specCentroid')
  comparisons[which(comparisons$Contrast=='Spectral centroid'), weights$sr[i]] = soundgen:::reportCI(quantile(coda_sr[, bName] * diff(range(sr$log_sc)) * 100, probs = c(.5, .025, .975)), digits = 1)
}
comparisons = comparisons[, c('Contrast', 'Afraid_weight', 'Aggressive_weight', 'Pleased_weight')]
write.csv(comparisons, '~/Downloads/temp_sr_extraPred.csv')
